----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    00:30:29 12/12/2011 
-- Design Name: 
-- Module Name:    stoper - mierz_czas 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity stoper is
	port (
		clk_i : IN std_logic;
		btn_state : IN std_logic;
		btn_rst : IN std_logic;
		led7_an_o : OUT std_logic_vector(3 downto 0);
		led7_seg_o : OUT std_logic_vector(7 downto 0)
	);
end stoper;

architecture mierz_czas of stoper is
	signal clk_10ms : std_logic := '0';
	signal digit_buf : std_logic_vector(31 downto 0):=X"81018181";
	signal tmp : std_logic := '0';
	type states is (Start, Stop);
	signal overload : std_logic := '0'; -- if overload = 1 then --.--	

	constant no0 : std_logic_vector(6 downto 0) := "0000001";
	constant no1 : std_logic_vector(6 downto 0) := "1001111";
	constant no2 : std_logic_vector(6 downto 0) := "0010010";
	constant no3 : std_logic_vector(6 downto 0) := "0000110";
	constant no4 : std_logic_vector(6 downto 0) := "1001100";
	constant no5 : std_logic_vector(6 downto 0) := "0100100";
	constant no6 : std_logic_vector(6 downto 0) := "0100000";
	constant no7 : std_logic_vector(6 downto 0) := "0001111";
	constant no8 : std_logic_vector(6 downto 0) := "0000000";
	constant no9 : std_logic_vector(6 downto 0) := "0000100";
	constant noA : std_logic_vector(6 downto 0) := "0001000";
	constant noB : std_logic_vector(6 downto 0) := "1100000";
	constant noC : std_logic_vector(6 downto 0) := "1110010";
	constant noD : std_logic_vector(6 downto 0) := "1000010";
	constant noE : std_logic_vector(6 downto 0) := "0110000";
	constant noF : std_logic_vector(6 downto 0) := "0111000";
	
	component divider_by_N is
		port (
			rst 		: IN Std_Logic;
			clk_in 	: IN Std_Logic;
			clk_out	: OUT Std_Logic
		);
   end component;
	
	component divider_by_N_10 is
		port (
			rst 		: IN Std_Logic;
			clk_in 	: IN Std_Logic;
			clk_out	: OUT Std_Logic
		);
   end component;
	
	component display_7seg is
		port (
			clk_i : IN std_logic;
			rst_i : IN std_logic;
			digit_i : IN std_logic_vector(31 downto 0);
			led7_an_o : OUT std_logic_vector(3 downto 0);
			led7_seg_o : OUT std_logic_vector(7 downto 0)
		);
	end component;
	
begin
	X1: divider_by_N port map (
		rst => '0',
		clk_in => clk_i,
		clk_out => tmp
	);
	
	X2: divider_by_N_10 port map (
		rst => '0',
		clk_in => clk_i,
		clk_out => clk_10ms
	);
	
   X3: display_7seg port map (
		clk_i => tmp,
		rst_i => btn_rst,
		digit_i => digit_buf,
		led7_an_o => led7_an_o,
		led7_seg_o => led7_seg_o
	);
	
	process(clk_10ms, btn_rst, btn_state, overload) is
	variable state : states;
	variable cnt_10ms : Natural range 0 to 9 := 0;
	variable cnt_100ms : Natural range 0 to 9 := 0;
	variable cnt_1s : Natural range 0 to 9 := 0;
	variable cnt_10s : Natural range 0 to 9 := 0;
	variable btn_state_buf:std_logic;  -- btn_state buffor contains last logic state of button "btn_state"
	
		begin
			if btn_rst = '1' then
				cnt_10ms := 0;
				cnt_100ms := 0;
				cnt_1s := 0;
				cnt_10s := 0;
				overload <= '0';
				digit_buf <= x"81018181";
			elsif overload = '1' then
				digit_buf<=X"FE7EFEFE";
			elsif rising_edge(clk_10ms) and overload='0' then
				if ( (btn_state = '1') and (btn_state_buf = '0') ) then	--rising edge trigger
					case (state) is
						when Start =>
							state := Stop;
							btn_state_buf := '1';
						when Stop =>
							state := Start;
							btn_state_buf := '1';
					end case;
				elsif btn_state = '0' then
					btn_state_buf := '0';
				end if;
				
				if ( state = Stop ) then
					cnt_10ms := cnt_10ms;
					cnt_100ms := cnt_100ms;
					cnt_1s := cnt_1s;
					cnt_10s := cnt_10s;
				elsif(state = Start) then
					if cnt_10ms < 9 then
						cnt_10ms := cnt_10ms + 1;
					else
						cnt_10ms := 0;
						if cnt_100ms < 9 then
							cnt_100ms := cnt_100ms + 1;
						else
							cnt_100ms := 0;
							if cnt_1s < 9 then
								cnt_1s := cnt_1s + 1;
							else
								cnt_1s := 0;
								if cnt_10s < 9 then
									cnt_10s := cnt_10s + 1;
								else
									overload <= '1';	
								end if; --end "if cnt_10s <= 5 then"
							end if; --end "if cnt_1s <= 9 then"
						end if; --end "if cnt_100ms <= 9 then"
					end if; --end "if cnt_10ms <= 9 then"
					case cnt_10ms is
						when 0 => digit_buf(6 downto 0)<=no0;
						when 1 => digit_buf(6 downto 0)<=no1;
						when 2 => digit_buf(6 downto 0)<=no2;
						when 3 => digit_buf(6 downto 0)<=no3;
						when 4 => digit_buf(6 downto 0)<=no4;
						when 5 => digit_buf(6 downto 0)<=no5;
						when 6 => digit_buf(6 downto 0)<=no6;
						when 7 => digit_buf(6 downto 0)<=no7;
						when 8 => digit_buf(6 downto 0)<=no8;
						when 9 => digit_buf(6 downto 0)<=no9;					
						when others=>digit_buf(31 downto 0)<=X"81018181";
					end case;
					
					case cnt_100ms is
						when 0 => digit_buf(14 downto 8)<=no0;
						when 1 => digit_buf(14 downto 8)<=no1;
						when 2 => digit_buf(14 downto 8)<=no2;
						when 3 => digit_buf(14 downto 8)<=no3;
						when 4 => digit_buf(14 downto 8)<=no4;
						when 5 => digit_buf(14 downto 8)<=no5;
						when 6 => digit_buf(14 downto 8)<=no6;
						when 7 => digit_buf(14 downto 8)<=no7;
						when 8 => digit_buf(14 downto 8)<=no8;
						when 9 => digit_buf(14 downto 8)<=no9;
						when others=>digit_buf(31 downto 0)<=X"81018181";
					end case; 
					
					case cnt_1s is
						when 0 => digit_buf(22 downto 16)<=no0;
						when 1 => digit_buf(22 downto 16)<=no1;
						when 2 => digit_buf(22 downto 16)<=no2;
						when 3 => digit_buf(22 downto 16)<=no3;
						when 4 => digit_buf(22 downto 16)<=no4;
						when 5 => digit_buf(22 downto 16)<=no5;
						when 6 => digit_buf(22 downto 16)<=no6;
						when 7 => digit_buf(22 downto 16)<=no7;
						when 8 => digit_buf(22 downto 16)<=no8;
						when 9 => digit_buf(22 downto 16)<=no9;
						when others=>digit_buf(31 downto 0)<=X"81018181";
					end case;  
					
					case cnt_10s is
						when 0 => digit_buf(30 downto 24)<=no0;
						when 1 => digit_buf(30 downto 24)<=no1;
						when 2 => digit_buf(30 downto 24)<=no2;
						when 3 => digit_buf(30 downto 24)<=no3;
						when 4 => digit_buf(30 downto 24)<=no4;
						when 5 => digit_buf(30 downto 24)<=no5;
						when 6 => digit_buf(30 downto 24)<=no6;
						when 7 => digit_buf(30 downto 24)<=no7;
						when 8 => digit_buf(30 downto 24)<=no8;
						when 9 => digit_buf(30 downto 24)<=no9;
						when others=>digit_buf(31 downto 0)<=X"81018181";
					end case;
				end if; --end "if ( state = Stop ) then"
			end if; --end "if btn_rst = '1' then"
	end process;
	
end mierz_czas;